# -*- coding: utf-8 -*- 
# @Time    : 04/03/2024 
# @Author  : nzooherd 
# @File    : redeem_code.py 
# @Software: Visual Studio Code

import hashlib
import string
import random

from exception.invalid_redeem import InvalidRedeemException


class RedeemCode:

    value2index = {1: 0, 5: 1, 10: 2, 20: 3, 50: 4, 100: 5}
    secretkey = "nzooherd"

    @staticmethod
    def encode(value: int) -> str:
        """
        giftcode: 10-10
        """
        prefix = ""
        digit_index = random.randint(0, 8)
        value_index = RedeemCode.value2index[value]
        mix_str =  RedeemCode.random_str()

        if digit_index == 0:
            prefix = str(value_index) + mix_str + str(digit_index)
        elif digit_index == 8:
            prefix = mix_str + str(value_index) + str(digit_index)
        else:
            prefix = mix_str[:digit_index] + str(value_index) + mix_str[digit_index:] + str(digit_index)
        
        suffix = hashlib.sha256((prefix+RedeemCode.secretkey).encode('utf-8')).hexdigest()[0:10]

        return f"{prefix}{suffix}"

    @staticmethod
    def decode(code: str) -> int:
        if len(code) != 20:
            raise InvalidRedeemException(code)
        
        prefix, suffix = code[0:10] , code[10:20]
        hash = hashlib.sha256((prefix+RedeemCode.secretkey).encode('utf-8')).hexdigest()[0:10]
        if not hash.startswith(suffix):
            raise InvalidRedeemException(code)

        digit_index = int(code[9])
        for value, index in RedeemCode.value2index.items():
            if index == int(prefix[digit_index]):
                return value

        raise InvalidRedeemException(code)

    @staticmethod
    def random_str(length=8):
        return ''.join(random.choices(string.ascii_letters + string.digits, k=8))


if __name__ == "__main__":
    for _ in range(10):
        code = RedeemCode.encode(5)
        print(code)